import unittest

from textwrap import dedent

from cedarpy_conversor import convert_json_to_cedar_policies

class ConvertJsonToPolicyTestCase(unittest.TestCase):
    def setUp(self) -> None:
        super().setUp()

    def test_policy_json_to_cedar(self):
        input_policy = dedent("""
            {
                "effect": "permit",
                "principal": {
                    "op": "==",
                    "entity": { "type": "User", "id": "12UA45" }
                },
                "action": {
                    "op": "==",
                    "entity": { "type": "Action", "id": "view" }
                },
                "resource": {
                    "op": "in",
                    "entity": { "type": "Folder", "id": "abc" }
                },
                "conditions": [
                    {
                        "kind": "when",
                        "body": {
                            "==": {
                                "left": {
                                    ".": {
                                        "left": {
                                            "Var": "context"
                                        },
                                        "attr": "tls_version"
                                    }
                                },
                                "right": {
                                    "Value": "1.3"
                                }
                            }
                        }
                    }
                ]
            }
        """).strip()

        expect_result = dedent("""
            permit(
              principal == User::"12UA45",
              action == Action::"view",
              resource in Folder::"abc"
            ) when {
              (context["tls_version"]) == "1.3"
            };
        """).strip()

        actual_result = convert_json_to_cedar_policies(input_policy)

        self.assertEqual(expect_result, actual_result)



import unittest

from textwrap import dedent

from cedarpy_conversor import get_entity_type_and_id

class GetEntityTypeAndIDnTestCase(unittest.TestCase):
    def setUp(self) -> None:
        super().setUp()

    def test_get_entity_type_and_id(self):
        input_entity = dedent("""
          Foo::Bar::"george"
        """).strip()

        expect_result = {'entity_id': 'george', 'entity_type': 'Foo::Bar'}
        actual_result = get_entity_type_and_id(input_entity)

        self.assertEqual(expect_result, actual_result.__dict__)



from cedarpy_conversor import _internal

def echo(s: str) -> str:
    return _internal.echo(s)

def convert_json_to_cedar_policies(policies_json_representation: str) -> str:
    """Convert the provided policies JSON representation to Cedar Policy.

    Parameters
    ----------
        * policies_json_representation: is a str containing the policies json representation to be converted

    Returns
    -------
        The converted policies (str)
        
    Raises
    -------
        ValueError: if the input policies_json_representation cannot be parsed or are not valid
    
    Examples
    -------
        ```python
        from cedarpy_conversor import convert_json_to_cedar_policies
        
        def main():
            json_representation_policies = "..."
            policies = convert_json_to_cedar_policies(json_representation_policies)
            print(policies)
        ```
    """
    return _internal.convert_json_to_cedar_policies(policies_json_representation)


def convert_cedar_policies_to_json(policies: str) -> str:
    """Convert the provided policies from Cedar Policy to JSON representation.

    Parameters
    ----------
        * policies: is a str containing the policies to be converted

    Returns
    -------
        The JSON representation of the policies (str)
        
    Raises
    -------
        ValueError: if the input policies cannot be parsed or are not valid
    
    Examples
    -------
        ```python
        from cedarpy_conversor import convert_cedar_policies_to_json
        
        def main():
            policies = "..."
            json_policies = convert_cedar_policies_to_json(policies)
            print(json_policies)
        ```
    """
    return _internal.convert_cedar_policies_to_json(policies)


class EntityTypeAndName:
   
    def __init__(self, response: dict):
        self._entity_type = response['entity_type']
        self._entity_id = response['entity_id']
    
    @property
    def entity_type(self) -> str:
        return self._entity_type
    
    @property
    def entity_id(self) -> str:
        return self._entity_id
    
    @property
    def __dict__(self):
        return {
            'entity_type': self.entity_type,
            'entity_id': self.entity_id
        }
    
def get_entity_type_and_id(entity: str) -> EntityTypeAndName:
    """Get the entity type and id from the provided entity.

    Parameters
    ----------
        * entity: is a str containing the entity to be parsed

    Returns
    -------
        The entity type and id (str)
        
    Raises
    -------
        ValueError: if the input entity cannot be parsed or are not valid
    
    Examples
    -------
        ```python
        from cedarpy_conversor import get_entity_type_and_id
        
        def main():
            entity = "..."
            entity_type_and_name = get_entity_type_and_id(entity)
            print(entity_type_and_name)
        ```
    """
    return EntityTypeAndName(_internal.get_entity_type_and_id(entity))


import unittest, json

from textwrap import dedent

from cedarpy_conversor import convert_cedar_policies_to_json

class ConvertPolicyToJsonTestCase(unittest.TestCase):
    def setUp(self) -> None:
        super().setUp()

    def test_cedar_policy_to_json(self):
        input_policy = dedent("""
            permit(
              principal == User::"12UA45",
              action == Action::"view",
              resource in Folder::"abc"
            ) when {
              (context["tls_version"]) == "1.3"
            };
        """).strip()

        expect_result = dedent("""{"effect":"permit","principal":{"op":"==","entity":{"type":"User","id":"12UA45"}},"action":{"op":"==","entity":{"type":"Action","id":"view"}},"resource":{"op":"in","entity":{"type":"Folder","id":"abc"}},"conditions":[{"kind":"when","body":{"==":{"left":{".":{"left":{"Var":"context"},"attr":"tls_version"}},"right":{"Value":"1.3"}}}}]}""")
        actual_result = convert_cedar_policies_to_json(input_policy)
        
        print(f"actual_result: {actual_result}")
        
        print(f"expect_result: {expect_result}")

        self.assertEqual(expect_result, actual_result)
